@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 """
     Created Oct 2008
-    TablePlotter Class
+    TableModel Class
     Copyright (C) Damien Farrell
- 
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
- 
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
- 
+
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -21,7 +21,9 @@
 
 from TableFormula import Formula
 from types import *
-import copy
+import operator
+import string, types, copy
+import pickle
 
 class TableModel(object):
     """A base model for managing the data in a TableCanvas class"""
@@ -34,8 +36,6 @@ class TableModel(object):
         """Constructor"""
         self.initialiseFields()
         self.setupModel(newdict, rows, columns)
-        #finally set default sort order as first col
-        #self.setSortOrder()
         return
 
     def setupModel(self, newdict, rows=None, columns=None):
@@ -56,7 +56,7 @@ class TableModel(object):
         else:
             #just make a new empty model
             self.createEmptyModel()
-       
+
         if not set(self.reclist) == set(self.data.keys()):
             print 'reclist does not match data keys'
         #restore last column order
@@ -69,15 +69,20 @@ class TableModel(object):
         #setup default display for column types
         self.default_display = {'text' : 'showstring',
                                 'number' : 'numtostring'}
+        #set default sort order as first col
+        if len(self.columnNames)>0:
+            self.sortkey = self.columnNames[0]
+        else:
+            self.sortkey = None
         #add rows and cols if they are given in the constructor
         if newdict == None:
             if rows != None:
-                self.auto_AddRows(rows)
+                self.autoAddRows(rows)
             if columns != None:
-                self.auto_AddColumns(columns)
-        self.filteredrecs = None          
+                self.autoAddColumns(columns)
+        self.filteredrecs = None
         return
-        
+
     def initialiseFields(self):
         """Create base fields, some of which are not saved"""
         self.data = None    # holds the table dict
@@ -106,43 +111,30 @@ class TableModel(object):
         self.reclist = self.data.keys()
         return
 
-    def importDict(self, newdata, namefield=None):
-        """Try to create a table model from some arbitrary dict"""
-        import types
-        if namefield == None:
-            namefield = 'name'
+    def importDict(self, newdata):
+        """Try to create a table model from a dict of the form
+           {{'rec1': {'col1': 3, 'col2': 2}, ..}"""
+
         #get cols from sub data keys
         colnames = []
-        colnames.append(namefield)
-        
         for k in newdata:
             fields = newdata[k].keys()
             for f in fields:
                 if not f in colnames:
                     colnames.append(f)
-
         for c in colnames:
             self.addColumn(c)
-
         #add the data
-        #print colnames
-        for k in newdata:
-            self.addRow(k)
-            for c in colnames:
-                if c == namefield:
-                    self.data[k][namefield] = str(k)
-                else:
-                    self.data[k][c] = str(newdata[k][c])
-
+        self.data.update(newdata)
+        self.reclist = self.data.keys()
         return
-    
+
     def getDefaultTypes(self):
         """Get possible field types for this table model"""
         return self.defaulttypes
 
     def getData(self):
         """Return the current data for saving"""
-        import copy
         data = copy.deepcopy(self.data)
         data['colors'] = self.colors
         data['columnnames'] = self.columnNames
@@ -174,7 +166,7 @@ class TableModel(object):
         collist = []
         if self.getColumnType(colIndex) == 'Link':
             return ['xxxxxx']
-        else:        
+        else:
             for row in range(len(self.reclist)):
                 v = self.getValueAt(row, colIndex)
                 collist.append(v)
@@ -182,10 +174,10 @@ class TableModel(object):
 
     def getlongestEntry(self, columnIndex):
         """Get the longest cell entry in the col"""
-        collist = self.getColCells(columnIndex)        
+        collist = self.getColCells(columnIndex)
         maxw=5
         for c in collist:
-            try:                
+            try:
                 w = len(str(c))
             except UnicodeEncodeError:
                 pass
@@ -205,7 +197,8 @@ class TableModel(object):
         value = None
         colname = self.getColumnName(columnIndex)
         coltype = self.columntypes[colname]
-        name = self.getRecName(rowIndex)    
+        name = self.getRecName(rowIndex)
+        #print self.data[name]
         if self.data[name].has_key(colname):
             celldata=self.data[name][colname]
         else:
@@ -238,10 +231,9 @@ class TableModel(object):
             return None
         currname = self.getRecName(rowIndex)
         self.reclist[rowIndex] = newname
-        import copy
         temp = copy.deepcopy(self.data[currname])
         self.data[newname] = temp
-        self.data[newname]['Name'] = newname
+        #self.data[newname]['Name'] = newname
         del self.data[currname]
         for key in ['bg', 'fg']:
             if self.colors[key].has_key(currname):
@@ -258,7 +250,7 @@ class TableModel(object):
          """Get the attribute of the record at the specified column index.
             This determines what will be displayed in the cell"""
 
-         value = None         
+         value = None
          if columnName != None and recName != None:
              if not self.data[recName].has_key(columnName):
                  return ''
@@ -273,8 +265,8 @@ class TableModel(object):
          if Formula.isFormula(cell) == True:
              value = self.doFormula(cell)
              return value
-           
-         if not type(cell) is DictType:             
+
+         if not type(cell) is DictType:
              if coltype == 'text' or coltype == 'Text':
                  value = cell
              elif coltype == 'number':
@@ -283,121 +275,119 @@ class TableModel(object):
                  value = 'other'
          if value==None:
              value=''
-
          return value
 
     def getRecordIndex(self, recname):
         rowIndex = self.reclist.index(recname)
         return rowIndex
 
-    def setSortOrder(self, columnIndex=0, reverse=0):
+    def setSortOrder(self, columnIndex=None, columnName=None, reverse=0):
         """Changes the order that records are sorted in, which will
            be reflected in the table upon redrawing"""
-        
-        self.sortcolumnIndex = columnIndex    
-        sortkey = self.getColumnName(columnIndex)
-        recnames = self.reclist
-        
-        self.reclist = self.createSortMap(self.reclist, sortkey, reverse)
+
+        if columnName != None and columnName in self.columnNames:
+            self.sortkey = columnName
+        elif columnIndex != None:
+            self.sortkey = self.getColumnName(columnIndex)
+        else:
+            return
+        self.reclist = self.createSortMap(self.reclist, self.sortkey, reverse)
         if self.filteredrecs != None:
-            self.filteredrecs = self.createSortMap(self.filteredrecs, sortkey, reverse)
+            self.filteredrecs = self.createSortMap(self.filteredrecs, self.sortkey, reverse)
         return
 
     def createSortMap(self, names, sortkey, reverse=0):
         """Create a sort mapping for given list"""
-        import operator
+
         recdata = []
-        for rec in names:            
+        for rec in names:
             recdata.append(self.getRecordAttributeAtColumn(recName=rec, columnName=sortkey))
         #try create list of floats if col has numbers only
-        try:            
+        try:
             recdata = self.toFloats(recdata)
         except:
             pass
         smap = zip(names, recdata)
         #sort the mapping by the second key
-        smap = sorted(smap, key=operator.itemgetter(1), reverse=reverse)        
+        smap = sorted(smap, key=operator.itemgetter(1), reverse=reverse)
         #now sort the main reclist by the mapping order
         sortmap = map(operator.itemgetter(0), smap)
         return sortmap
-    
+
     def toFloats(self, l):
         x=[]
         for i in l:
             if i == '':
                 x.append(0.0)
-            else:    
+            else:
                 x.append(float(i))
         return x
-    
-    def getSortIndex(self):
+
+    '''def getSortIndex(self):
         """Return the current sort order index"""
         if self.sortcolumnIndex:
             return self.sortcolumnIndex
         else:
-            return 0
+            return 0'''
 
     def moveColumn(self, oldcolumnIndex, newcolumnIndex):
         """Changes the order of columns"""
         self.oldnames = self.columnNames
         self.columnNames=[]
-        #self.columnOrder=[]
 
-        print oldcolumnIndex, newcolumnIndex
         #write out a new column names list - tedious
         moved = self.oldnames[oldcolumnIndex]
         del self.oldnames[oldcolumnIndex]
-        print self.oldnames
+        #print self.oldnames
         i=0
         for c in self.oldnames:
             if i==newcolumnIndex:
                 self.columnNames.append(moved)
-                #self.columnOrder.append(moved)
             self.columnNames.append(c)
-            #self.columnOrder.append(c)
             i=i+1
         #if new col is at end just append
         if moved not in self.columnNames:
             self.columnNames.append(moved)
-            #self.columnOrder.append(moved)
-
-        print self.columnNames
-  
         return
 
-    def addRow(self, name=None, **kwargs):
+    def getNextKey(self):
+        """Return the next numeric key in the dict"""
+        num = len(self.reclist)+1
+        return num
+
+    def addRow(self, key=None, **kwargs):
         """Add a row"""
-        if self.data.has_key(name) or name in self.reclist:
+        if key == '':
+            return
+        if key==None:
+            key = self.getNextKey()
+        if self.data.has_key(key) or key in self.reclist:
             print 'name already present!!'
             return
-        self.data[name]={}
-        if name != None:
-            self.data[name]['Name'] = name
-        else:
-            self.data[name]['Name'] = ''
-        self.reclist.append(name)       
+        self.data[key]={}
+        for k in kwargs:
+            if not k in self.columnNames:
+                self.addColumn(k)
+            self.data[key][k] = str(kwargs[k])
+        self.reclist.append(key)
+        return key
 
-        return
-
-    def deleteRow(self, rowIndex, update=True):
+    def deleteRow(self, rowIndex=None, key=None, update=True):
         """Delete a row"""
-        name = self.getRecName(rowIndex)
-        del self.data[name]
+        if key == None or not key in self.reclist:
+            key = self.getRecName(rowIndex)
+        del self.data[key]
         if update==True:
-            self.reclist = self.data.keys()
-
+            self.reclist.remove(key)
         return
 
     def deleteRows(self, rowlist=None):
         """Delete multiple or all rows"""
         if rowlist == None:
             rowlist = range(len(self.reclist))
-        #print 'deleting' , rowlist
-        #print 'reclist', self.reclist
-        for row in rowlist:
-            self.deleteRow(row, update=False)
-        self.reclist = self.data.keys()
-
+        names = [self.getRecName(i) for i in rowlist]
+        for name in names:
+            self.deleteRow(key=name, update=True)
         return
 
     def addColumn(self, colname=None, coltype=None):
@@ -414,13 +404,11 @@ class TableModel(object):
             self.columntypes[colname]='text'
         else:
             self.columntypes[colname]=coltype
-
         return
 
     def deleteColumn(self, columnIndex):
         """delete a column"""
         colname = self.getColumnName(columnIndex)
-        print colname
         self.columnNames.remove(colname)
         del self.columnlabels[colname]
         del self.columntypes[colname]
@@ -428,49 +416,40 @@ class TableModel(object):
         for recname in self.reclist:
             if self.data[recname].has_key(colname):
                 del self.data[recname][colname]
-
-        if hasattr(self, 'sortcolumnIndex') and columnIndex == self.sortcolumnIndex:
-            self.setSortOrder()
-        print 'column deleted'
-        print 'new cols:', self.columnNames
+        if self.sortkey != None:
+            currIndex = self.getColumnIndex(self.sortkey)
+            if columnIndex == currIndex:
+                self.setSortOrder(0)
+        #print 'column deleted'
+        #print 'new cols:', self.columnNames
         return
 
-    def deleteMultipleColumns(self, cols=None):
+    def deleteColumns(self, cols=None):
         """Remove all cols or list provided"""
-
-        while self.getColumnCount() > 0:
-            self.deleteColumn(0)
+        if cols == None:
+            cols = self.columnNames
+        if self.getColumnCount() == 0:
+            return
+        for col in cols:
+            self.deleteColumn(col)
         return
 
-    def auto_AddRows(self, numrows=None):
+    def autoAddRows(self, numrows=None):
         """Automatically add x number of records"""
-        import string
-        alphabet = string.lowercase[:26]
         rows = self.getRowCount()
-
-        if rows <= 25:
-            i=rows
-            j=0
+        ints = [i for i in self.reclist if isinstance(i, int)]
+        if len(ints)>0:
+            start = max(ints)+1
         else:
-            i=int(rows%25)
-            j=int(round(rows/25,1))
-        #print i, j
-        for x in range(numrows):
-            if i >= len(alphabet):
-                i=0
-                j=j+1
-            name = alphabet[i]+str(j)
-            if name in self.reclist:
-                pass
-            else:
-                self.addRow(name)
-            i=i+1
-            #print self.reclist
-        return
+            start = 0
+        keys = range(start,start+numrows)
+        for k in keys:
+            self.addRow(k)
+        return keys
 
-    def auto_AddColumns(self, numcols=None):
+    def autoAddColumns(self, numcols=None):
         """Automatically add x number of cols"""
-        import string
+
         alphabet = string.lowercase[:26]
         currcols=self.getColumnCount()
         #find where to start
@@ -504,7 +483,7 @@ class TableModel(object):
          return len(self.columnNames)
 
     def getColumnName(self, columnIndex):
-         """Returns the name of the given column by columnIndex."""        
+         """Returns the name of the given column by columnIndex."""
          return self.columnNames[columnIndex]
 
     def getColumnLabel(self, columnIndex):
@@ -522,7 +501,7 @@ class TableModel(object):
         """Return the data in a list for this col,
             filterby is a tuple of key, value that allows to filter by rec attribute
         """
-        import types
+
         if columnIndex != None:
             columnName = self.getColumnName(columnIndex)
         coldata = []
@@ -537,80 +516,81 @@ class TableModel(object):
                     else:
                         coldata.append(self.data[r][columnName])
         else:
-            for r in self.reclist:                                  
+            for r in self.reclist:
                 if not self.data[r].has_key(columnName):
                     coldata.append('')
                 else:
                     coldata.append(self.data[r][columnName])
         return coldata
-        
+
     def getColumns(self, colnames, filterby=None, allowempty=True):
         """Get column data for multiple cols, with given filter options,
             filterby: a tuple of key value that allows filtering
             allowempty: boolean if false means rows with empty vals for any
             required fields are not returned
             returns: lists of column data"""
+
         def evaluate(l):
             for i in l:
                 if i == '' or i == None:
                     return False
-            return True        
-          
-        coldata=[]       
+            return True
+
+        coldata=[]
         for c in colnames:
-            vals = self.getColumnData(columnName=c, filterby=filterby)            
-            coldata.append(vals)          
-        if allowempty == False:  
-            result = [i for i in zip(*coldata) if evaluate(i) == True]               
-            coldata = zip(*result)        
+            vals = self.getColumnData(columnName=c, filterby=filterby)
+            coldata.append(vals)
+        if allowempty == False:
+            result = [i for i in zip(*coldata) if evaluate(i) == True]
+            coldata = zip(*result)
         return coldata
-        
-    def getDict(self, colnames, filterby=None):  
+
+    def getDict(self, colnames, filterby=None):
         """Get the model data as a dict for given columns with filter options"""
         data={}
-        names, = self.getColumns(['name'], filterby)        
+        names, = self.getColumns(['name'], filterby)
         cols = self.getColumns(colnames, filterby)
-        coldata = zip(*cols)   
-        for name,cdata in zip(names, coldata):           
+        coldata = zip(*cols)
+        for name,cdata in zip(names, coldata):
             data[name] = dict(zip(colnames,cdata))
         return data
-        
+
     def filterBy(self, filtercol, value, op='contains', userecnames=False,
                      progresscallback=None):
         """Filter recs according to one column"""
-        
-        from Filtering import Operators        
+
+        from Filtering import Operators
         funcs = {'contains':Operators.contains,'=':Operators.equals,
                    '>':Operators.greaterthan,'<':Operators.lessthan,
                    'starts with':Operators.startswith,
                    'ends with':Operators.endswith,
                    'has length':Operators.haslength}
-        floatops = ['=','>','<']                   
+        floatops = ['=','>','<']
         func = funcs[op]
         data = self.data
         #coltype = self.columntypes[filtercol]
         names=[]
         for rec in self.reclist:
-            if data[rec].has_key(filtercol):  
+            if data[rec].has_key(filtercol):
                 #try to do float comparisons if required
                 if op in floatops:
                     try:
                         item = float(data[rec][filtercol])
                         v=float(value)
-                        if func(v, item) == True:                    
+                        if func(v, item) == True:
                             names.append(rec)
-                        continue 
+                        continue
                     except:
                         pass
                 if filtercol == 'name' and userecnames == True:
                     item = rec
                 else:
-                    item = str(data[rec][filtercol])                
-                if func(value, item):                    
-                    names.append(rec)               
-            
-        return names        
-        
+                    item = str(data[rec][filtercol])
+                if func(value, item):
+                    names.append(rec)
+
+        return names
+
     def filterByExpr(self, expr):
         """Filter recs using simple expression"""
         import shlex
@@ -635,7 +615,7 @@ class TableModel(object):
 
     def getValueAt(self, rowIndex, columnIndex):
          """Returns the cell value at location specified
-             by columnIndex and rowIndex."""        
+             by columnIndex and rowIndex."""
          value = self.getRecordAttributeAtColumn(rowIndex, columnIndex)
          return value
 
@@ -644,7 +624,7 @@ class TableModel(object):
         name = self.getRecName(rowIndex)
         colname = self.getColumnName(columnIndex)
         coltype = self.columntypes[colname]
-      
+
         if coltype == 'number':
             try:
                 if value == '': #need this to allow deletion of values
@@ -654,7 +634,7 @@ class TableModel(object):
             except:
                 pass
         else:
-            self.data[name][colname] = value       
+            self.data[name][colname] = value
         return
 
     def setFormulaAt(self, f, rowIndex, columnIndex):
@@ -682,7 +662,7 @@ class TableModel(object):
         colname = self.getColumnName(columnIndex)
         if not self.colors[key].has_key(name):
             self.colors[key][name] = {}
-        self.colors[key][name][colname] = str(color)        
+        self.colors[key][name][colname] = str(color)
         return
 
     def resetcolors(self):
@@ -749,8 +729,8 @@ class TableModel(object):
         return newformula
 
     def merge(self, model, key='name', fields=None):
-        """Merge another table model with this one based on a key field, 
-           we only add nrecords from the new model where the key is present
+        """Merge another table model with this one based on a key field,
+           we only add records from the new model where the key is present
            in both models"""
         if fields == None: fields = model.columnNames
         for rec in self.reclist:
@@ -760,48 +740,50 @@ class TableModel(object):
                 if not model.data[new].has_key(key):
                     continue
                 if self.data[rec][key] == model.data[new][key]:
-                #if new == rec:                                   
+                #if new == rec:
                     for f in fields:
                         if not model.data[rec].has_key(f):
                             continue
                         if not f in self.columnNames:
-                            self.addColumn(f)                        
-                        self.data[rec][f] = model.data[rec][f]       
+                            self.addColumn(f)
+                        self.data[rec][f] = model.data[rec][f]
         return
-        
-    def addRecord(self, name, **kwargs):
+
+    '''def addRecord(self, name, **kwargs):
         """Add new rec with all fields in kwargs dict"""
         #name = kwargs[kwargs.keys()[0]]
-        self.addRow(name) 
+        self.addRow(name)
         self.addColumn('name')
-        self.data[name]['name'] = name          
-        for k in kwargs:            
+        self.data[name]['name'] = name
+        for k in kwargs:
             if not k in self.columnNames:
                 self.addColumn(k)
             self.data[name][k] = str(kwargs[k])
-        return
+        return'''
 
     def save(self, filename=None):
         """Save model to file"""
         if filename == None:
             return
         data = self.getData()
-        fd=open(filename,'w')
-        import pickle
+        fd = open(filename,'w')
         pickle.dump(data,fd)
         fd.close()
         return
 
-    def load(self):
-        """Load model from pickle file"""        
+    def load(self, filename):
+        """Load model from pickle file"""
+        fd=open(filename,'r')
+        data = pickle.load(fd)
+        self.setupModel(data)
         return
 
     def copy(self):
         """Return a copy of this model"""
-        M=TableModel()
+        M = TableModel()
         data = self.getData()
         M.setupModel(data)
         return M
-    
+
     def __repr__(self):
         return 'Table Model with %s rows' %len(self.reclist)
